package javax.swing.plaf.basic;

import java.awt.Container;
import javax.swing.plaf.InputMapUIResource;
import java.awt.event.ActionEvent;
import sun.swing.UIAction;
import java.beans.PropertyChangeEvent;
import sun.awt.AppContext;
import javax.swing.plaf.ComponentUI;
import javax.swing.InputMap;
import javax.swing.KeyStroke;
import javax.swing.plaf.ComponentInputMapUIResource;
import javax.swing.LookAndFeel;
import java.awt.Component;
import java.awt.Font;
import java.awt.Dimension;
import java.awt.Insets;
import javax.swing.text.View;
import javax.swing.JComponent;
import java.awt.Color;
import sun.swing.SwingUtilities2;
import java.awt.Graphics;
import javax.swing.SwingUtilities;
import javax.swing.Icon;
import java.awt.FontMetrics;
import javax.swing.JLabel;
import java.awt.Rectangle;
import java.beans.PropertyChangeListener;
import javax.swing.plaf.LabelUI;

public class BasicLabelUI extends LabelUI implements PropertyChangeListener
{
    protected static BasicLabelUI labelUI;
    private static final Object BASIC_LABEL_UI_KEY;
    private Rectangle paintIconR;
    private Rectangle paintTextR;
    
    public BasicLabelUI() {
        this.paintIconR = new Rectangle();
        this.paintTextR = new Rectangle();
    }
    
    static void loadActionMap(LazyActionMap lazyActionMap) {
        lazyActionMap.put(new BasicLabelUI.Actions("press"));
        lazyActionMap.put(new BasicLabelUI.Actions("release"));
    }
    
    protected String layoutCL(JLabel c, FontMetrics fm, String text, Icon icon, Rectangle viewR, Rectangle iconR, Rectangle textR) {
        return SwingUtilities.layoutCompoundLabel(c, fm, text, icon, c.getVerticalAlignment(), c.getHorizontalAlignment(), c.getVerticalTextPosition(), c.getHorizontalTextPosition(), viewR, iconR, textR, c.getIconTextGap());
    }
    
    protected void paintEnabledText(JLabel c, Graphics g, String text, int x, int y) {
        int displayedMnemonicIndex = c.getDisplayedMnemonicIndex();
        g.setColor(c.getForeground());
        SwingUtilities2.drawStringUnderlineCharAt(c, g, text, displayedMnemonicIndex, x, y);
    }
    
    protected void paintDisabledText(JLabel label, Graphics graphics, String s, int x, int y) {
        int displayedMnemonicIndex = label.getDisplayedMnemonicIndex();
        Color background = label.getBackground();
        graphics.setColor(background.brighter());
        SwingUtilities2.drawStringUnderlineCharAt(label, graphics, s, displayedMnemonicIndex, x + 1, y + 1);
        graphics.setColor(background.darker());
        SwingUtilities2.drawStringUnderlineCharAt(label, graphics, s, displayedMnemonicIndex, x, y);
    }
    
    @Override
    public void paint(Graphics g, JComponent component) {
        JLabel c = (JLabel)component;
        String text = c.getText();
        Icon icon = c.isEnabled() ? c.getIcon() : c.getDisabledIcon();
        if (icon == null && text == null) {
            return;
        }
        FontMetrics fontMetrics = SwingUtilities2.getFontMetrics(c, g);
        String layout = this.layout(c, fontMetrics, component.getWidth(), component.getHeight());
        if (icon != null) {
            icon.paintIcon(component, g, this.paintIconR.x, this.paintIconR.y);
        }
        if (text != null) {
            View view = (View)component.getClientProperty("html");
            if (view != null) {
                view.paint(g, this.paintTextR);
            }
            else {
                int x = this.paintTextR.x;
                int n = this.paintTextR.y + fontMetrics.getAscent();
                if (c.isEnabled()) {
                    this.paintEnabledText(c, g, layout, x, n);
                }
                else {
                    this.paintDisabledText(c, g, layout, x, n);
                }
            }
        }
        return;
    }
    
    private String layout(JLabel label, FontMetrics fontMetrics, int n, int n2) {
        Insets insets = label.getInsets(null);
        String text = label.getText();
        Icon icon = label.isEnabled() ? label.getIcon() : label.getDisabledIcon();
        Rectangle rectangle = new Rectangle();
        rectangle.x = insets.left;
        rectangle.y = insets.top;
        rectangle.width = n - (insets.left + insets.right);
        rectangle.height = n2 - (insets.top + insets.bottom);
        Rectangle paintIconR = this.paintIconR;
        Rectangle paintIconR2 = this.paintIconR;
        Rectangle paintIconR3 = this.paintIconR;
        Rectangle paintIconR4 = this.paintIconR;
        int n3 = 0;
        paintIconR4.height = n3;
        paintIconR3.width = n3;
        paintIconR2.y = n3;
        paintIconR.x = n3;
        Rectangle paintTextR = this.paintTextR;
        Rectangle paintTextR2 = this.paintTextR;
        Rectangle paintTextR3 = this.paintTextR;
        Rectangle paintTextR4 = this.paintTextR;
        int n4 = 0;
        paintTextR4.height = n4;
        paintTextR3.width = n4;
        paintTextR2.y = n4;
        paintTextR.x = n4;
        return this.layoutCL(label, fontMetrics, text, icon, rectangle, this.paintIconR, this.paintTextR);
    }
    
    @Override
    public Dimension getPreferredSize(JComponent component) {
        JLabel label = (JLabel)component;
        String text = label.getText();
        Icon icon = label.isEnabled() ? label.getIcon() : label.getDisabledIcon();
        Insets insets = label.getInsets(null);
        Font font = label.getFont();
        int n = insets.left + insets.right;
        int n2 = insets.top + insets.bottom;
        if (icon == null && (text == null || (text != null && font == null))) {
            return new Dimension(n, n2);
        }
        if (text == null || (icon != null && font == null)) {
            return new Dimension(icon.getIconWidth() + n, icon.getIconHeight() + n2);
        }
        FontMetrics fontMetrics = label.getFontMetrics(font);
        Rectangle rectangle = new Rectangle();
        Rectangle rectangle2 = new Rectangle();
        Rectangle rectangle3 = new Rectangle();
        Rectangle rectangle4 = rectangle;
        int n3 = 0;
        rectangle.height = n3;
        rectangle.width = n3;
        rectangle.y = n3;
        rectangle4.x = n3;
        Rectangle rectangle5 = rectangle2;
        int n4 = 0;
        rectangle2.height = n4;
        rectangle2.width = n4;
        rectangle2.y = n4;
        rectangle5.x = n4;
        rectangle3.x = n;
        rectangle3.y = n2;
        Rectangle rectangle6 = rectangle3;
        int n5 = 32767;
        rectangle3.height = n5;
        rectangle6.width = n5;
        this.layoutCL(label, fontMetrics, text, icon, rectangle3, rectangle, rectangle2);
        Dimension dimension = new Dimension(Math.max(rectangle.x + rectangle.width, rectangle2.x + rectangle2.width) - Math.min(rectangle.x, rectangle2.x), Math.max(rectangle.y + rectangle.height, rectangle2.y + rectangle2.height) - Math.min(rectangle.y, rectangle2.y));
        dimension.width += n;
        dimension.height += n2;
        return dimension;
    }
    
    @Override
    public Dimension getMinimumSize(JComponent component) {
        Dimension preferredSize = this.getPreferredSize(component);
        View view = (View)component.getClientProperty("html");
        if (view != null) {
            preferredSize.width = (int)((float)preferredSize.width - (view.getPreferredSpan(0) - view.getMinimumSpan(0)));
            return preferredSize;
        }
        return preferredSize;
    }
    
    @Override
    public Dimension getMaximumSize(JComponent component) {
        Dimension preferredSize = this.getPreferredSize(component);
        View view = (View)component.getClientProperty("html");
        if (view != null) {
            preferredSize.width = (int)((float)preferredSize.width + (view.getMaximumSpan(0) - view.getPreferredSpan(0)));
            return preferredSize;
        }
        return preferredSize;
    }
    
    @Override
    public int getBaseline(JComponent c, int width, int height) {
        super.getBaseline(c, width, height);
        JLabel label = (JLabel)c;
        String text = label.getText();
        if (text == null || "".equals(text) || label.getFont() == null) {
            return -1;
        }
        FontMetrics fontMetrics = label.getFontMetrics(label.getFont());
        this.layout(label, fontMetrics, width, height);
        return BasicHTML.getBaseline(label, this.paintTextR.y, fontMetrics.getAscent(), this.paintTextR.width, this.paintTextR.height);
    }
    
    @Override
    public Component.BaselineResizeBehavior getBaselineResizeBehavior(JComponent c) {
        super.getBaselineResizeBehavior(c);
        if (c.getClientProperty("html") != null) {
            return Component.BaselineResizeBehavior.OTHER;
        }
        switch (((JLabel)c).getVerticalAlignment()) {
            case 1: {
                return Component.BaselineResizeBehavior.CONSTANT_ASCENT;
            }
            case 3: {
                return Component.BaselineResizeBehavior.CONSTANT_DESCENT;
            }
            case 0: {
                return Component.BaselineResizeBehavior.CENTER_OFFSET;
            }
            default: {
                return Component.BaselineResizeBehavior.OTHER;
            }
        }
    }
    
    @Override
    public void installUI(JComponent component) {
        this.installDefaults((JLabel)component);
        this.installComponents((JLabel)component);
        this.installListeners((JLabel)component);
        this.installKeyboardActions((JLabel)component);
    }
    
    @Override
    public void uninstallUI(JComponent component) {
        this.uninstallDefaults((JLabel)component);
        this.uninstallComponents((JLabel)component);
        this.uninstallListeners((JLabel)component);
        this.uninstallKeyboardActions((JLabel)component);
    }
    
    protected void installDefaults(JLabel label) {
        LookAndFeel.installColorsAndFont(label, "Label.background", "Label.foreground", "Label.font");
        LookAndFeel.installProperty(label, "opaque", Boolean.FALSE);
    }
    
    protected void installListeners(JLabel label) {
        label.addPropertyChangeListener(this);
    }
    
    protected void installComponents(JLabel label) {
        BasicHTML.updateRenderer(label, label.getText());
        label.setInheritsPopupMenu(true);
    }
    
    protected void installKeyboardActions(JLabel label) {
        int displayedMnemonic = label.getDisplayedMnemonic();
        Component label2 = label.getLabelFor();
        if (displayedMnemonic != 0 && label2 != null) {
            LazyActionMap.installLazyActionMap(label, BasicLabelUI.class, "Label.actionMap");
            InputMap uiInputMap2 = SwingUtilities.getUIInputMap(label, 2);
            if (uiInputMap2 == null) {
                uiInputMap2 = new ComponentInputMapUIResource(label);
                SwingUtilities.replaceUIInputMap(label, 2, uiInputMap2);
            }
            uiInputMap2.clear();
            uiInputMap2.put(KeyStroke.getKeyStroke(displayedMnemonic, BasicLookAndFeel.getFocusAcceleratorKeyMask(), false), "press");
        }
        else {
            InputMap uiInputMap = SwingUtilities.getUIInputMap(label, 2);
            if (uiInputMap != null) {
                uiInputMap.clear();
            }
        }
    }
    
    protected void uninstallDefaults(JLabel label) {
    }
    
    protected void uninstallListeners(JLabel label) {
        label.removePropertyChangeListener(this);
    }
    
    protected void uninstallComponents(JLabel label) {
        BasicHTML.updateRenderer(label, "");
    }
    
    protected void uninstallKeyboardActions(JLabel component) {
        SwingUtilities.replaceUIInputMap(component, 0, null);
        SwingUtilities.replaceUIInputMap(component, 2, null);
        SwingUtilities.replaceUIActionMap(component, null);
    }
    
    public static ComponentUI createUI(JComponent component) {
        if (System.getSecurityManager() != null) {
            AppContext appContext = AppContext.getAppContext();
            BasicLabelUI value = (BasicLabelUI)appContext.get(BasicLabelUI.BASIC_LABEL_UI_KEY);
            if (value == null) {
                value = new BasicLabelUI();
                appContext.put(BasicLabelUI.BASIC_LABEL_UI_KEY, value);
                return value;
            }
            return value;
        }
        else {
            return BasicLabelUI.labelUI;
        }
    }
    
    @Override
    public void propertyChange(PropertyChangeEvent propertyChangeEvent) {
        String propertyName = propertyChangeEvent.getPropertyName();
        if (propertyName == "text" || "font" == propertyName || "foreground" == propertyName) {
            JLabel label = (JLabel)propertyChangeEvent.getSource();
            BasicHTML.updateRenderer(label, label.getText());
        }
        else if (propertyName == "labelFor" || propertyName == "displayedMnemonic") {
            this.installKeyboardActions((JLabel)propertyChangeEvent.getSource());
        }
    }
    
    static {
        BasicLabelUI.labelUI = new BasicLabelUI();
        BASIC_LABEL_UI_KEY = new Object();
    }
    
    private static class Actions extends UIAction
    {
        private static final String PRESS = "press";
        private static final String RELEASE = "release";
        
        Actions(String name) {
            super(name);
        }
        
        @Override
        public void actionPerformed(ActionEvent actionEvent) {
            JLabel label = (JLabel)actionEvent.getSource();
            String name = this.getName();
            if (name == "press") {
                this.doPress(label);
            }
            else if (name == "release") {
                this.doRelease(label, actionEvent.getActionCommand() != null);
            }
        }
        
        private void doPress(JLabel label) {
            Component label2 = label.getLabelFor();
            if (label2 != null && label2.isEnabled()) {
                InputMap uiInputMap = SwingUtilities.getUIInputMap(label, 0);
                if (uiInputMap == null) {
                    uiInputMap = new InputMapUIResource();
                    SwingUtilities.replaceUIInputMap(label, 0, uiInputMap);
                }
                int displayedMnemonic = label.getDisplayedMnemonic();
                this.putOnRelease(uiInputMap, displayedMnemonic, BasicLookAndFeel.getFocusAcceleratorKeyMask());
                this.putOnRelease(uiInputMap, displayedMnemonic, 0);
                this.putOnRelease(uiInputMap, 18, 0);
                label.requestFocus();
            }
        }
        
        private void doRelease(JLabel label, boolean b) {
            Component label2 = label.getLabelFor();
            if (label2 != null && label2.isEnabled()) {
                if (label.hasFocus()) {
                    InputMap uiInputMap2 = SwingUtilities.getUIInputMap(label, 0);
                    if (uiInputMap2 != null) {
                        int displayedMnemonic2 = label.getDisplayedMnemonic();
                        this.removeOnRelease(uiInputMap2, displayedMnemonic2, BasicLookAndFeel.getFocusAcceleratorKeyMask());
                        this.removeOnRelease(uiInputMap2, displayedMnemonic2, 0);
                        this.removeOnRelease(uiInputMap2, 18, 0);
                    }
                    InputMap uiInputMap3 = SwingUtilities.getUIInputMap(label, 2);
                    if (uiInputMap3 == null) {
                        uiInputMap3 = new InputMapUIResource();
                        SwingUtilities.replaceUIInputMap(label, 2, uiInputMap3);
                    }
                    int displayedMnemonic3 = label.getDisplayedMnemonic();
                    if (b) {
                        this.putOnRelease(uiInputMap3, 18, 0);
                    }
                    else {
                        this.putOnRelease(uiInputMap3, displayedMnemonic3, BasicLookAndFeel.getFocusAcceleratorKeyMask());
                        this.putOnRelease(uiInputMap3, displayedMnemonic3, 0);
                    }
                    if (label2 instanceof Container && ((Container)label2).isFocusCycleRoot()) {
                        label2.requestFocus();
                    }
                    else {
                        SwingUtilities2.compositeRequestFocus(label2);
                    }
                }
                else {
                    InputMap uiInputMap = SwingUtilities.getUIInputMap(label, 2);
                    int displayedMnemonic = label.getDisplayedMnemonic();
                    if (uiInputMap != null) {
                        if (b) {
                            this.removeOnRelease(uiInputMap, displayedMnemonic, BasicLookAndFeel.getFocusAcceleratorKeyMask());
                            this.removeOnRelease(uiInputMap, displayedMnemonic, 0);
                        }
                        else {
                            this.removeOnRelease(uiInputMap, 18, 0);
                        }
                    }
                }
            }
        }
        
        private void putOnRelease(InputMap inputMap, int keyCode, int modifiers) {
            inputMap.put(KeyStroke.getKeyStroke(keyCode, modifiers, true), "release");
        }
        
        private void removeOnRelease(InputMap inputMap, int keyCode, int modifiers) {
            inputMap.remove(KeyStroke.getKeyStroke(keyCode, modifiers, true));
        }
    }
}
